# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

CAR_COLOR_RECORD = {'White', 'Silver', 'Black', 'Grey', 'Blue', 'Red', 'Brown', 'Green'}
CAR_MAKE_RECORD = {'McLaren', 'Koenigsegg', 'Bugatti', 'Pagani', 'Aston Martin', 'Mercedes', 'Jaguar', 'Porsche',
            'Ferrari', 'Lamborghini', 'Rolls Royce', 'Bentley', 'Maybach', 'Audi', 'Maserati', 'Lexus', 'BMW', 'Cadillac'}
CAR_STYLE = {'R': 'Roadster', 'CV': 'Convertible', 'CP': 'Coupe', 'S': 'Sedan', 'SUV': 'Sport utility vehicle'}
CAR_GENRE = {'M': 'Modern', 'C': 'Classic', 'A': 'Antique', 'E': 'Exotic'}
CAR_RARITY = {'super rare', 'very rare', 'rare', 'fairly rare', None}
CAR_CONDITION = {'DC': 'Driving Condition', 'SS': 'Street Show Condition', 'SC': 'Show Car', 'CC': 'Concours Condition'}
CAR_PETROL_GRADE = {87: 'regular', 88: 'midgrade', 89: 'midgrade',
                    90: 'midgrade', 91: 'premium', 92: 'premium', 93: 'premium', 94: 'premium'}

CRYPTOCURRENCY_PROOF_TYPES = {'Proof of Work', 'Proof of Stake'}

PRICING_METHOD = {'Derived', 'Market'}

PRIVATE_INVESTMENT_CATEGORY = {'Private Equity', 'Mutual Funds', 'Hedge Funds', 'Fund of Funds', 'StartUp', 'Private Company', 'Others'}
PRIVATE_INVESTMENT_SUBCATEGORY = {'Mutual Funds': {'Money Market Funds', 'Bond Funds', 'Balanced Funds', 'Equity Funds',
                                                   'Speciality Funds'},
                                  'Private Equity': {'Leverage Buyout Funds', 'Growth Equity Funds', 'Venture Capital Funds',
                                                     'Real Estate Investment Funds', 'Mezzanine Funds', 'Distresses Debt Funds',
                                                     'Special Situation Funds'},
                                  'Hedge Funds': {'Market Neutral Funds', 'Equity Long Funds', 'Equity Short Funds', 'Event-Driven Funds'
                                                  'Macro Funds', 'Currency Funds', 'Equity Funds', 'Commodity', 'Credit', 'Index Arbitrage', 
                                                  'Quant Funds'},
                                  'Fund of Funds': {'Market Neutral Funds', 'Equity Long Funds', 'Equity Short Funds', 'Event-Driven Funds'
                                                    'Macro Funds', 'Currency Funds', 'Equity Funds', 'Commodity', 'Credit', 'Index Arbitrage', 
                                                    'Quant Funds'},
                                  'StartUp': {'Lifestyle Startup', 'Small Business', 'Silicon Valley Type Startup', 'Startup to be Quickly Sold'
                                              'Large Company Startup', 'Social Startups'}}
PRIVATE_INVESTMENT_SHARE_TYPE = {'Ordinary Shares', 'Defered Ordinary Shares', 'Non-voting Ordinary Shares', 'Redeemable Shares',
                                 'Redeemable Participating Shares', 'Redeemable Preference Shares', 'Preference Shares',
                                 'Cumulative Preference Shares', 'Participating Prefered Shares', 'Prefered Convertible Shares',
                                 'Prefered Convertible Shares', 'Convertible Note', 'Loan'}

WINE_CLASSIFICATIONS = {'1er Cru', '1er Cru Classé', '1er Grand Cru', '2ème Cru Classé', '2ème Vin',
                        '4ème Cru Classé', 'Cru Classé', 'Grand Cru', 'Non Classé'}
WINE_PACKING_TYPE = {'OCB06', 'OCB12', 'OWC06', 'OWC01', 'OWC02', 'OWC03', 'OWC06', 'OWC12'}


